import argparse
from concurrent import futures
import sys
import grpc

sys.path.insert(0, "../../../../../flatbuffers/python")

import flatbuffers
from models import HelloReply, HelloRequest, greeter_grpc_fb

parser = argparse.ArgumentParser()
parser.add_argument("port", help="server on port", default=3000)


def build_reply(message):
  builder = flatbuffers.Builder()
  ind = builder.CreateString(message)
  HelloReply.HelloReplyStart(builder)
  HelloReply.HelloReplyAddMessage(builder, ind)
  root = HelloReply.HelloReplyEnd(builder)
  builder.Finish(root)
  return bytes(builder.Output())


class GreeterServicer(greeter_grpc_fb.GreeterServicer):

  def __init__(self):
    self.greetings = ["Hi", "Hallo", "Ciao"]

  def SayHello(self, request, context):
    r = HelloRequest.HelloRequest().GetRootAs(request, 0)
    reply = "Unknown"
    if r.Name():
      reply = r.Name()
    return build_reply("welcome " + reply.decode("UTF-8"))

  def SayManyHellos(self, request, context):
    r = HelloRequest.HelloRequest().GetRootAs(request, 0)
    reply = "Unknown"
    if r.Name():
      reply = r.Name()

    for greeting in self.greetings:
      print(type(reply))
      yield build_reply(greeting + " " + reply.decode("UTF-8"))


def serve():
  args = parser.parse_args()
  server = grpc.server(futures.ThreadPoolExecutor(max_workers=10))
  greeter_grpc_fb.add_GreeterServicer_to_server(GreeterServicer(), server)
  server.add_insecure_port("[::]:" + args.port)
  server.start()
  server.wait_for_termination()


if __name__ == "__main__":
  serve()



# automatically generated by the FlatBuffers compiler, do not modify

# namespace: Example

import flatbuffers
from flatbuffers.compat import import_numpy
np = import_numpy()

class Test(object):
    __slots__ = ['_tab']

    @classmethod
    def SizeOf(cls):
        return 4

    # Test
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # Test
    def A(self): return self._tab.Get(flatbuffers.number_types.Int16Flags, self._tab.Pos + flatbuffers.number_types.UOffsetTFlags.py_type(0))
    # Test
    def B(self): return self._tab.Get(flatbuffers.number_types.Int8Flags, self._tab.Pos + flatbuffers.number_types.UOffsetTFlags.py_type(2))

def CreateTest(builder, a, b):
    builder.Prep(2, 4)
    builder.Pad(1)
    builder.PrependInt8(b)
    builder.PrependInt16(a)
    return builder.Offset()


class TestT(object):

    # TestT
    def __init__(
        self,
        a = 0,
        b = 0,
    ):
        self.a = a  # type: int
        self.b = b  # type: int

    @classmethod
    def InitFromBuf(cls, buf, pos):
        test = Test()
        test.Init(buf, pos)
        return cls.InitFromObj(test)

    @classmethod
    def InitFromPackedBuf(cls, buf, pos=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, pos)
        return cls.InitFromBuf(buf, pos+n)

    @classmethod
    def InitFromObj(cls, test):
        x = TestT()
        x._UnPack(test)
        return x

    # TestT
    def _UnPack(self, test):
        if test is None:
            return
        self.a = test.A()
        self.b = test.B()

    # TestT
    def Pack(self, builder):
        return CreateTest(builder, self.a, self.b)



#!/usr/bin/env python3
#
# Copyright 2022 Google Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys

from flatc_cpp_tests import CppTests
from flatc_kotlin_tests import KotlinTests
from flatc_schema_tests import SchemaTests
from flatc_test import run_all
from flatc_ts_tests import TsTests

passing, failing = run_all(CppTests, KotlinTests, TsTests, SchemaTests)

print("")
print("{0} of {1} tests passed".format(passing, passing + failing))

if failing > 0:
  sys.exit(1)



# automatically generated by the FlatBuffers compiler, do not modify

# namespace: NestedUnion

import flatbuffers
from flatbuffers.compat import import_numpy
from typing import Any
from MyGame.Example.NestedUnion.Test import Test
from typing import Optional
np = import_numpy()

class Vec3(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAs(cls, buf, offset: int = 0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = Vec3()
        x.Init(buf, n + offset)
        return x

    @classmethod
    def GetRootAsVec3(cls, buf, offset=0):
        """This method is deprecated. Please switch to GetRootAs."""
        return cls.GetRootAs(buf, offset)
    # Vec3
    def Init(self, buf: bytes, pos: int):
        self._tab = flatbuffers.table.Table(buf, pos)

    # Vec3
    def X(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Float64Flags, o + self._tab.Pos)
        return 0.0

    # Vec3
    def Y(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Float64Flags, o + self._tab.Pos)
        return 0.0

    # Vec3
    def Z(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Float64Flags, o + self._tab.Pos)
        return 0.0

    # Vec3
    def Test1(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(10))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Float64Flags, o + self._tab.Pos)
        return 0.0

    # Vec3
    def Test2(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(12))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Uint8Flags, o + self._tab.Pos)
        return 0

    # Vec3
    def Test3(self) -> Optional[Test]:
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(14))
        if o != 0:
            x = o + self._tab.Pos
            obj = Test()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

def Vec3Start(builder: flatbuffers.Builder):
    builder.StartObject(6)

def Start(builder: flatbuffers.Builder):
    Vec3Start(builder)

def Vec3AddX(builder: flatbuffers.Builder, x: float):
    builder.PrependFloat64Slot(0, x, 0.0)

def AddX(builder: flatbuffers.Builder, x: float):
    Vec3AddX(builder, x)

def Vec3AddY(builder: flatbuffers.Builder, y: float):
    builder.PrependFloat64Slot(1, y, 0.0)

def AddY(builder: flatbuffers.Builder, y: float):
    Vec3AddY(builder, y)

def Vec3AddZ(builder: flatbuffers.Builder, z: float):
    builder.PrependFloat64Slot(2, z, 0.0)

def AddZ(builder: flatbuffers.Builder, z: float):
    Vec3AddZ(builder, z)

def Vec3AddTest1(builder: flatbuffers.Builder, test1: float):
    builder.PrependFloat64Slot(3, test1, 0.0)

def AddTest1(builder: flatbuffers.Builder, test1: float):
    Vec3AddTest1(builder, test1)

def Vec3AddTest2(builder: flatbuffers.Builder, test2: int):
    builder.PrependUint8Slot(4, test2, 0)

def AddTest2(builder: flatbuffers.Builder, test2: int):
    Vec3AddTest2(builder, test2)

def Vec3AddTest3(builder: flatbuffers.Builder, test3: Any):
    builder.PrependStructSlot(5, flatbuffers.number_types.UOffsetTFlags.py_type(test3), 0)

def AddTest3(builder: flatbuffers.Builder, test3: Any):
    Vec3AddTest3(builder, test3)

def Vec3End(builder: flatbuffers.Builder) -> int:
    return builder.EndObject()

def End(builder: flatbuffers.Builder) -> int:
    return Vec3End(builder)

import MyGame.Example.NestedUnion.Test
try:
    from typing import Optional
except:
    pass

class Vec3T(object):

    # Vec3T
    def __init__(
        self,
        x = 0.0,
        y = 0.0,
        z = 0.0,
        test1 = 0.0,
        test2 = 0,
        test3 = None,
    ):
        self.x = x  # type: float
        self.y = y  # type: float
        self.z = z  # type: float
        self.test1 = test1  # type: float
        self.test2 = test2  # type: int
        self.test3 = test3  # type: Optional[MyGame.Example.NestedUnion.Test.TestT]

    @classmethod
    def InitFromBuf(cls, buf, pos):
        vec3 = Vec3()
        vec3.Init(buf, pos)
        return cls.InitFromObj(vec3)

    @classmethod
    def InitFromPackedBuf(cls, buf, pos=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, pos)
        return cls.InitFromBuf(buf, pos+n)

    @classmethod
    def InitFromObj(cls, vec3):
        x = Vec3T()
        x._UnPack(vec3)
        return x

    # Vec3T
    def _UnPack(self, vec3):
        if vec3 is None:
            return
        self.x = vec3.X()
        self.y = vec3.Y()
        self.z = vec3.Z()
        self.test1 = vec3.Test1()
        self.test2 = vec3.Test2()
        if vec3.Test3() is not None:
            self.test3 = MyGame.Example.NestedUnion.Test.TestT.InitFromObj(vec3.Test3())

    # Vec3T
    def Pack(self, builder):
        Vec3Start(builder)
        Vec3AddX(builder, self.x)
        Vec3AddY(builder, self.y)
        Vec3AddZ(builder, self.z)
        Vec3AddTest1(builder, self.test1)
        Vec3AddTest2(builder, self.test2)
        if self.test3 is not None:
            test3 = self.test3.Pack(builder)
            Vec3AddTest3(builder, test3)
        vec3 = Vec3End(builder)
        return vec3



#!/usr/bin/env python
"""
Build script for vendored LMDB with CFFI.

This script:
1. Copies LMDB C sources from lmdb-upstream/ submodule to build directory
2. Applies patches from lmdb-patches/
3. Generates zlmdb/lmdb/_config.py for CFFI compilation
4. Configures CFFI to build the LMDB extension

Based on py-lmdb setup.py patch application logic.
"""

import os
import sys
import shutil
import subprocess
import platform


def apply_patches():
    """Copy LMDB sources and apply patches."""
    print("=" * 70)
    print("Building vendored LMDB with CFFI")
    print("=" * 70)

    # Source and destination paths
    lmdb_src = os.path.join('lmdb-upstream', 'libraries', 'liblmdb')
    build_dir = os.path.join('build', 'lmdb-src')
    patches_dir = 'lmdb-patches'

    if not os.path.exists(lmdb_src):
        print(f"ERROR: LMDB source not found at {lmdb_src}")
        print("Did you initialize git submodules?")
        print("Run: git submodule update --init --recursive")
        sys.exit(1)

    # Create build directory
    os.makedirs('build', exist_ok=True)

    # Clean old build directory
    if os.path.exists(build_dir):
        print(f"Removing old build directory: {build_dir}")
        shutil.rmtree(build_dir)

    # Copy LMDB sources to build directory
    print(f"Copying LMDB sources from {lmdb_src} to {build_dir}")
    shutil.copytree(lmdb_src, build_dir)

    # Apply patches
    patch_files = [
        'env-copy-txn.patch',
        'its-10346.patch',
    ]

    for patch_file in patch_files:
        patch_path = os.path.join(patches_dir, patch_file)
        if not os.path.exists(patch_path):
            print(f"WARNING: Patch file not found: {patch_path}")
            continue

        print(f"Applying patch: {patch_file}")

        # The patches are from py-lmdb which has structure:
        # libraries/liblmdb/...
        # We need to apply with -p3 to strip those path components

        if sys.platform.startswith('win'):
            try:
                import patch_ng as patch

                # Use absolute paths for patch-ng
                abs_patch_path = os.path.abspath(patch_path)
                abs_build_dir = os.path.abspath(build_dir)

                # Debug info
                print(f"  Patch file: {abs_patch_path}")
                print(f"  Build directory: {abs_build_dir}")
                print(f"  Build directory exists: {os.path.exists(abs_build_dir)}")
                if os.path.exists(abs_build_dir):
                    print(f"  Files in build dir: {os.listdir(abs_build_dir)[:5]}")

                patchset = patch.fromfile(abs_patch_path)
                if not patchset:
                    print(f"ERROR: Failed to parse patch file {patch_file}")
                    sys.exit(1)

                print(f"  Parsed {len(patchset.items)} patch items")

                # Debug: show what we're trying to patch
                for item in patchset.items:
                    target = item.target if isinstance(item.target, str) else item.target.decode('utf-8', errors='replace')
                    print(f"    Patch item target (raw): {target}")

                # List files in build directory to verify they exist
                print(f"  Files in {abs_build_dir}:")
                for f in os.listdir(abs_build_dir)[:10]:
                    print(f"    - {f}")

                # Apply patch - try strip=3 first, fallback to auto-detect
                # The patches have paths like: a/libraries/liblmdb/lmdb.h
                # We need to strip 3 levels: a/, libraries/, liblmdb/
                # to get: lmdb.h (which is directly in build/lmdb-src/)
                print("  Attempting to apply with strip=3...")
                success = False

                # Try strip levels 0-4 until one works
                for strip_level in [3, 2, 4, 1, 0]:
                    print(f"  Trying strip={strip_level}...")
                    try:
                        # patch-ng.PatchSet.apply(strip=N, root=path)
                        result = patchset.apply(strip_level, abs_build_dir)
                        if result:
                            print(f"  [OK] Successfully applied with strip={strip_level}")
                            success = True
                            break
                        else:
                            print(f"  [FAIL] strip={strip_level} did not work")
                    except Exception as e:
                        print(f"  [ERROR] strip={strip_level} raised: {e}")
                        continue

                if not success:
                    # None of the strip levels worked
                    print(f"  [FAIL] Failed to apply patch {patch_file} with any strip level")
                    print(f"ERROR: Failed to apply patch {patch_file}")
                    sys.exit(1)
            except ImportError:
                print("ERROR: patch-ng module required on Windows")
                print("Install with: pip install patch-ng")
                sys.exit(1)
            except Exception as e:
                print(f"ERROR: Exception while applying patch {patch_file}: {e}")
                import traceback
                traceback.print_exc()
                sys.exit(1)
        else:
            # Unix: use patch command
            cmd = ['patch', '-N', '-p3', '-d', build_dir, '-i',
                   os.path.abspath(patch_path)]
            result = subprocess.run(cmd, capture_output=True, text=True)
            if result.returncode != 0:
                # patch returns non-zero if already applied, check output
                if 'Reversed (or previously applied) patch detected' in result.stdout:
                    print("  (already applied)")
                else:
                    print(f"ERROR: Failed to apply patch {patch_file}")
                    print(result.stdout)
                    print(result.stderr)
                    sys.exit(1)
            else:
                print("  [OK] Applied successfully")

    return build_dir


def generate_config(build_dir):
    """Generate src/zlmdb/_lmdb_vendor/_config.py with build configuration."""
    print("\nGenerating src/zlmdb/_lmdb_vendor/_config.py")

    # Source files to compile
    extra_sources = [
        os.path.join(build_dir, 'mdb.c'),
        os.path.join(build_dir, 'midl.c'),
    ]

    # Include directories
    extra_include_dirs = [
        build_dir,
        'lmdb-patches',  # for preload.h
    ]

    # Compile args
    extra_compile_args = [
        '-UNDEBUG',  # Disable NDEBUG
        '-DHAVE_PATCHED_LMDB=1',  # Mark as patched version
    ]

    # Disable warnings unless maintainer mode
    if not os.getenv('LMDB_MAINTAINER'):
        extra_compile_args.append('-w')

    # Windows-specific configuration
    libraries = []
    if sys.platform.startswith('win'):
        # Add Windows-specific includes for MSVC compatibility
        extra_include_dirs.append('lmdb-patches')
        libraries.append('Advapi32')

        # MSVC version detection
        p = sys.version.find('MSC v.')
        msvc_ver = int(sys.version[p + 6: p + 10]) if p != -1 else None
        if msvc_ver and not msvc_ver >= 1600:
            # Visual Studio <= 2010 needs stdint.h emulation
            print("  (adding stdint.h emulation for old MSVC)")

    # Write configuration
    config = {
        'extra_compile_args': extra_compile_args,
        'extra_sources': extra_sources,
        'extra_library_dirs': [],
        'extra_include_dirs': extra_include_dirs,
        'libraries': libraries,
    }

    config_file = os.path.join('src', 'zlmdb', '_lmdb_vendor', '_config.py')
    print(f"  Writing to {config_file}")

    with open(config_file, 'w') as f:
        f.write('# Auto-generated by build_lmdb.py\n')
        f.write('# DO NOT EDIT - regenerated on each build\n\n')
        f.write('CONFIG = {\n')
        for key, value in config.items():
            f.write(f'    {key!r}: {value!r},\n')
        f.write('}\n')

    print("  [OK] Configuration written")
    return config


def main():
    """Main build entry point."""
    print("\n" + "=" * 70)
    print("zlmdb LMDB build script")
    print("=" * 70)
    print(f"Python: {sys.version}")
    print(f"Platform: {platform.platform()}")
    print(f"Working directory: {os.getcwd()}")
    print()

    # Step 1: Apply patches
    build_dir = apply_patches()

    # Step 2: Generate config
    config = generate_config(build_dir)

    print("\n" + "=" * 70)
    print("LMDB build preparation complete")
    print("=" * 70)
    print("\nCFFI will now compile the patched LMDB sources.")
    print("This happens automatically during 'pip install' or 'python -m build'")
    print()

    return 0


if __name__ == '__main__':
    sys.exit(main())



# automatically generated by the FlatBuffers compiler, do not modify

# namespace: goldens

import flatbuffers
from flatbuffers.compat import import_numpy
np = import_numpy()

class Galaxy(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAs(cls, buf, offset=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = Galaxy()
        x.Init(buf, n + offset)
        return x

    @classmethod
    def GetRootAsGalaxy(cls, buf, offset=0):
        """This method is deprecated. Please switch to GetRootAs."""
        return cls.GetRootAs(buf, offset)
    # Galaxy
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # Galaxy
    def NumStars(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Int64Flags, o + self._tab.Pos)
        return 0

def GalaxyStart(builder):
    builder.StartObject(1)

def Start(builder):
    GalaxyStart(builder)

def GalaxyAddNumStars(builder, numStars):
    builder.PrependInt64Slot(0, numStars, 0)

def AddNumStars(builder, numStars):
    GalaxyAddNumStars(builder, numStars)

def GalaxyEnd(builder):
    return builder.EndObject()

def End(builder):
    return GalaxyEnd(builder)



# automatically generated by the FlatBuffers compiler, do not modify

# namespace: MyGame

import flatbuffers
from flatbuffers.compat import import_numpy
np = import_numpy()

class InParentNamespace(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAs(cls, buf, offset=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = InParentNamespace()
        x.Init(buf, n + offset)
        return x

    @classmethod
    def GetRootAsInParentNamespace(cls, buf, offset=0):
        """This method is deprecated. Please switch to GetRootAs."""
        return cls.GetRootAs(buf, offset)
    @classmethod
    def InParentNamespaceBufferHasIdentifier(cls, buf, offset, size_prefixed=False):
        return flatbuffers.util.BufferHasIdentifier(buf, offset, b"\x4D\x4F\x4E\x53", size_prefixed=size_prefixed)

    # InParentNamespace
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

def InParentNamespaceStart(builder):
    builder.StartObject(0)

def Start(builder):
    InParentNamespaceStart(builder)

def InParentNamespaceEnd(builder):
    return builder.EndObject()

def End(builder):
    return InParentNamespaceEnd(builder)


class InParentNamespaceT(object):

    # InParentNamespaceT
    def __init__(
        self,
    ):
        pass

    @classmethod
    def InitFromBuf(cls, buf, pos):
        inParentNamespace = InParentNamespace()
        inParentNamespace.Init(buf, pos)
        return cls.InitFromObj(inParentNamespace)

    @classmethod
    def InitFromPackedBuf(cls, buf, pos=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, pos)
        return cls.InitFromBuf(buf, pos+n)

    @classmethod
    def InitFromObj(cls, inParentNamespace):
        x = InParentNamespaceT()
        x._UnPack(inParentNamespace)
        return x

    # InParentNamespaceT
    def _UnPack(self, inParentNamespace):
        if inParentNamespace is None:
            return

    # InParentNamespaceT
    def Pack(self, builder):
        InParentNamespaceStart(builder)
        inParentNamespace = InParentNamespaceEnd(builder)
        return inParentNamespace



###############################################################################
#
# The MIT License (MIT)
#
# Copyright (c) typedef int GmbH
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
###############################################################################

import os
import sys
import pytest
import logging
from copy import deepcopy

import txaio

txaio.use_twisted()

import zlmdb  # noqa

try:
    from tempfile import TemporaryDirectory
except ImportError:
    from backports.tempfile import TemporaryDirectory  # type:ignore

sys.path.append(os.path.dirname(os.path.abspath(__file__)))

if sys.version_info >= (3, 6):
    logging.info("Using _schema_py3 !")
    from _schema_py3 import User, Schema4
else:
    logging.info("Using _schema_py2 !")
    from _schema_py2 import User, Schema4


@pytest.fixture(scope="function")
def testset1(N=10, M=100):
    users = []
    for j in range(N):
        for i in range(M):
            user = User.create_test_user(oid=j * M + i, realm_oid=j)
            users.append(user)
    return users


def test_fill_indexes(testset1):
    """
    Fill a table with multiple indexes with data records that have all columns filled
    with NON-NULL values.
    """
    with TemporaryDirectory() as dbpath:
        schema = Schema4()

        with zlmdb.Database(dbpath) as db:
            stats = zlmdb.TransactionStats()

            # fill table, which also triggers inserts into the index pmaps
            with db.begin(write=True, stats=stats) as txn:
                for user in testset1:
                    schema.users[txn, user.oid] = user

            # check indexes has been written to (in addition to the table itself)
            num_indexes = len(schema.users.indexes())
            assert stats.puts == len(testset1) * (1 + num_indexes)

            # check saved objects
            with db.begin() as txn:
                for user in testset1:
                    obj = schema.users[txn, user.oid]

                    assert user == obj

            # check unique indexes
            with db.begin() as txn:
                for user in testset1:
                    user_oid = schema.idx_users_by_authid[txn, user.authid]
                    assert user.oid == user_oid

                    user_oid = schema.idx_users_by_email[txn, user.email]
                    assert user.oid == user_oid

                    user_oid = schema.idx_users_by_icecream[
                        txn, (user.icecream, user.oid)
                    ]
                    assert user.oid == user_oid

                    user_oid = schema.idx_users_by_mrealm_authid[
                        txn, (user.mrealm, user.authid)
                    ]
                    assert user.oid == user_oid

                    user_oid = schema.idx_users_by_mrealm_notnull_authid[
                        txn, (user.mrealm_notnull, user.authid)
                    ]
                    assert user.oid == user_oid

            # check non-unique index
            users_by_icecream = {}
            for user in testset1:
                if user.icecream not in users_by_icecream:
                    users_by_icecream[user.icecream] = set()
                users_by_icecream[user.icecream].add(user.oid)

            MAX_OID = 9007199254740992
            with db.begin() as txn:
                for icecream in users_by_icecream:
                    for _, user_oid in schema.idx_users_by_icecream.select(
                        txn,
                        from_key=(icecream, 0),
                        to_key=(icecream, MAX_OID + 1),
                        return_values=False,
                    ):
                        assert user_oid in users_by_icecream[icecream]


def test_fill_indexes_nullable(testset1):
    """
    Test filling a table with multiple indexes, some of which are on NULLable columns, and fill
    with records that have those column values actually NULL.
    """
    with TemporaryDirectory() as dbpath:
        schema = Schema4()

        with zlmdb.Database(dbpath) as db:
            stats = zlmdb.TransactionStats()

            with db.begin(write=True, stats=stats) as txn:
                for user in testset1:
                    _user = deepcopy(user)

                    # "user.email" is an indexed column that is nullable
                    _user.email = None

                    # "user.mrealm" is an indexed (composite) column that is nullable
                    _user.mrealm = None
                    schema.users[txn, _user.oid] = _user

            # check indexes has been written to (in addition to the table itself)
            num_indexes = len(schema.users.indexes())

            # because we have set 2 indexed columns to NULL, we need to subtract those 2
            # from the total number of indexes
            assert stats.puts == len(testset1) * (1 + num_indexes - 2)

            # check saved objects
            with db.begin() as txn:
                for user in testset1:
                    _user = deepcopy(user)
                    _user.email = None
                    _user.mrealm = None

                    obj = schema.users[txn, _user.oid]

                    assert _user == obj

            # check unique indexes
            with db.begin() as txn:
                for user in testset1:
                    # check one of the indexes that was indeed filled
                    user_oid = schema.idx_users_by_authid[txn, user.authid]
                    assert user.oid == user_oid

                    # check indexes that have NOT been filled
                    user_oid = schema.idx_users_by_email[txn, user.email]
                    assert user_oid is None

                    user_oid = schema.idx_users_by_mrealm_authid[
                        txn, (user.mrealm, user.authid)
                    ]
                    assert user_oid is None


def test_fill_index_non_nullable_raises(testset1):
    """
    Insert records into a table with a unique-non-nullable index with the
    record having a NULL value in the indexed column raises an exception.
    """
    with TemporaryDirectory() as dbpath:
        schema = Schema4()

        with zlmdb.Database(dbpath) as db:
            with db.begin(write=True) as txn:
                for user in testset1:
                    # "user.authid" is an indexed column that is non-nullable
                    _user = deepcopy(user)
                    _user.authid = None
                    with pytest.raises(zlmdb.NullValueConstraint):
                        schema.users[txn, _user.oid] = _user

                    # "user.mrealm_notnull" is an indexed (composite) column that is non-nullable
                    _user = deepcopy(user)
                    _user.mrealm_notnull = None
                    with pytest.raises(zlmdb.NullValueConstraint):
                        schema.users[txn, _user.oid] = _user


def test_fill_non_unique_indexes(testset1):
    """
    Insert records into a table with a non-unique, non-nullable indexed column.
    """
    with TemporaryDirectory() as dbpath:
        schema = Schema4()

        with zlmdb.Database(dbpath) as db:
            stats = zlmdb.TransactionStats()

            with db.begin(write=True, stats=stats) as txn:
                for user in testset1:
                    schema.users[txn, user.oid] = user

            # check non-unique indexes
            with db.begin() as txn:
                for j in range(10):
                    user_oids = list(
                        schema.idx_users_by_realm.select(
                            txn, return_keys=False, from_key=(j, 0), to_key=(j + 1, 0)
                        )
                    )

                    assert list(range(j * 100, (j + 1) * 100)) == user_oids


def test_delete_indexes(testset1):
    """
    Insert records into a table with indexes, delete data records and check that index
    records have been deleted as a consequence too.
    """
    with TemporaryDirectory() as dbpath:
        schema = Schema4()

        with zlmdb.Database(dbpath) as db:
            stats = zlmdb.TransactionStats()

            # insert data records
            with db.begin(write=True, stats=stats) as txn:
                for user in testset1:
                    schema.users[txn, user.oid] = user

            # check that all index records have been deleted as well
            with db.begin(write=True) as txn:
                for user in testset1:
                    del schema.users[txn, user.oid]

                    user_oid = schema.idx_users_by_authid[txn, user.authid]
                    assert user_oid is None

                    user_oid = schema.idx_users_by_email[txn, user.email]
                    assert user_oid is None

                    user_oid = schema.idx_users_by_realm[
                        txn, (user.realm_oid, user.oid)
                    ]
                    assert user_oid is None

                    user_oid = schema.idx_users_by_icecream[
                        txn, (user.icecream, user.oid)
                    ]
                    assert user_oid is None

                    user_oid = schema.idx_users_by_mrealm_authid[
                        txn, (user.mrealm, user.authid)
                    ]
                    assert user_oid is None

                    user_oid = schema.idx_users_by_mrealm_notnull_authid[
                        txn, (user.mrealm_notnull, user.authid)
                    ]
                    assert user_oid is None


def test_delete_nonunique_indexes(testset1):
    """
    Insert records into a table with a non-unique index, delete data records and check
    that index records have been deleted as a consequence too.
    """
    with TemporaryDirectory() as dbpath:
        schema = Schema4()

        with zlmdb.Database(dbpath) as db:
            stats = zlmdb.TransactionStats()

            with db.begin(write=True, stats=stats) as txn:
                for user in testset1:
                    schema.users[txn, user.oid] = user

            with db.begin(write=True) as txn:
                for user in testset1:
                    del schema.users[txn, user.oid]

            with db.begin() as txn:
                for j in range(10):
                    user_oids = list(
                        schema.idx_users_by_realm.select(
                            txn, return_keys=False, from_key=(j, 0), to_key=(j + 1, 0)
                        )
                    )

                    assert [] == user_oids


def test_delete_nonindexes2(testset1):
    """

    WARNING: quadratic run-time (in testset size)
    """
    with TemporaryDirectory() as dbpath:
        schema = Schema4()

        with zlmdb.Database(dbpath) as db:
            stats = zlmdb.TransactionStats()

            with db.begin(write=True, stats=stats) as txn:
                for user in testset1:
                    schema.users[txn, user.oid] = user

            with db.begin(write=True) as txn:
                for j in range(10):
                    fullset = set(range(j * 100, (j + 1) * 100))
                    for i in range(100):
                        user_oid = j * 100 + i
                        del schema.users[txn, user_oid]
                        fullset.discard(user_oid)

                        user_oids = set(
                            schema.idx_users_by_realm.select(
                                txn,
                                return_keys=False,
                                from_key=(j, 0),
                                to_key=(j + 1, 0),
                            )
                        )

                        assert fullset == user_oids


def test_set_null_indexes_nullable(testset1):
    """
    Fill table with indexed column (unique-nullable) with indexed column
    values NULL, then (in a 2nd transaction) set the indexed column
    to NON-NULL value and check that index records are deleted.
    """
    with TemporaryDirectory() as dbpath:
        schema = Schema4()

        with zlmdb.Database(dbpath) as db:
            stats = zlmdb.TransactionStats()

            # fill table with NON-NULLs in indexed column
            with db.begin(write=True) as txn:
                for user in testset1:
                    schema.users[txn, user.oid] = user

            # now update table with NULLs in indexed column
            with db.begin(write=True, stats=stats) as txn:
                for user in testset1:
                    _user = schema.users[txn, user.oid]
                    _user.email = None
                    schema.users[txn, _user.oid] = _user

            # check that the table records have their indexed
            # column values updated to NULLs
            with db.begin() as txn:
                for user in testset1:
                    _user = deepcopy(user)
                    _user.email = None
                    obj = schema.users[txn, user.oid]
                    assert _user == obj

            # check that the index records that previously existed
            # have been deleted (as the indexed column values have been
            # set to NULLs)
            with db.begin() as txn:
                for user in testset1:
                    user_oid = schema.idx_users_by_authid[txn, user.authid]
                    assert user.oid == user_oid

                    user_oid = schema.idx_users_by_email[txn, user.email]
                    assert user_oid is None


def test_set_notnull_indexes_nullable(testset1):
    """
    Fill table with indexed column (unique-nullable) with indexed column
    values NON-NULL, then (in a 2nd transaction) set the indexed column
    to NULL value and check that index records are created.
    """
    with TemporaryDirectory() as dbpath:
        schema = Schema4()

        with zlmdb.Database(dbpath) as db:
            stats = zlmdb.TransactionStats()

            # fill table with NULLs in indexed column
            with db.begin(write=True) as txn:
                for user in testset1:
                    _user = deepcopy(user)
                    _user.email = None
                    schema.users[txn, _user.oid] = _user

            # now update table with NON-NULLs in indexed column
            with db.begin(write=True, stats=stats) as txn:
                for user in testset1:
                    _user = schema.users[txn, user.oid]
                    _user.email = user.email
                    schema.users[txn, _user.oid] = _user

            # check that the table records have their indexed
            # column values updated to NON-NULLs
            with db.begin() as txn:
                for user in testset1:
                    obj = schema.users[txn, user.oid]
                    assert user == obj

            # check that the index records that previously not existed
            # have been created (as the indexed column values have been
            # set to NON-NULLs)
            with db.begin() as txn:
                for user in testset1:
                    user_oid = schema.idx_users_by_authid[txn, user.authid]
                    assert user.oid == user_oid

                    user_oid = schema.idx_users_by_email[txn, user.email]
                    assert user.oid == user_oid


def test_truncate_table_with_index(testset1):
    """
    Fill a table with records that has indexes, truncate the table and check that
    all index records have been deleted as well.
    """
    with TemporaryDirectory() as dbpath:
        schema = Schema4()

        with zlmdb.Database(dbpath) as db:
            with db.begin(write=True) as txn:
                for user in testset1:
                    schema.users[txn, user.oid] = user

        stats = zlmdb.TransactionStats()

        with zlmdb.Database(dbpath) as db:
            with db.begin(write=True, stats=stats) as txn:
                records = schema.users.truncate(txn)

                assert records == len(testset1) * (len(schema.users.indexes()) + 1)

        assert stats.dels == records
        assert stats.puts == 0



from __future__ import annotations

import flatbuffers
import numpy as np

import typing
from MyGame.Example.TestEnum import TestEnum

uoffset: typing.TypeAlias = flatbuffers.number_types.UOffsetTFlags.py_type

class NestedStruct(object):
  @classmethod
  def SizeOf(cls) -> int: ...

  def Init(self, buf: bytes, pos: int) -> None: ...
  def A(self, i: int) -> typing.List[int]: ...
  def AAsNumpy(self) -> np.ndarray: ...
  def ALength(self) -> int: ...
  def AIsNone(self) -> bool: ...
  def B(self) -> typing.Literal[TestEnum.A, TestEnum.B, TestEnum.C]: ...
  def C(self, i: int) -> typing.Literal[TestEnum.A, TestEnum.B, TestEnum.C]: ...
  def CAsNumpy(self) -> np.ndarray: ...
  def CLength(self) -> int: ...
  def CIsNone(self) -> bool: ...
  def D(self, i: int) -> typing.List[int]: ...
  def DAsNumpy(self) -> np.ndarray: ...
  def DLength(self) -> int: ...
  def DIsNone(self) -> bool: ...
class NestedStructT(object):
  a: typing.List[int]
  b: typing.Literal[TestEnum.A, TestEnum.B, TestEnum.C]
  c: typing.List[typing.Literal[TestEnum.A, TestEnum.B, TestEnum.C]]
  d: typing.List[int]
  def __init__(
    self,
    a: typing.List[int] | None = ...,
    b: typing.Literal[TestEnum.A, TestEnum.B, TestEnum.C] = ...,
    c: typing.List[typing.Literal[TestEnum.A, TestEnum.B, TestEnum.C]] | None = ...,
    d: typing.List[int] | None = ...,
  ) -> None: ...
  @classmethod
  def InitFromBuf(cls, buf: bytes, pos: int) -> NestedStructT: ...
  @classmethod
  def InitFromPackedBuf(cls, buf: bytes, pos: int = 0) -> NestedStructT: ...
  @classmethod
  def InitFromObj(cls, nestedStruct: NestedStruct) -> NestedStructT: ...
  def _UnPack(self, nestedStruct: NestedStruct) -> None: ...
  def Pack(self, builder: flatbuffers.Builder) -> None: ...

def CreateNestedStruct(builder: flatbuffers.Builder, a: int, b: typing.Literal[TestEnum.A, TestEnum.B, TestEnum.C], c: typing.Literal[TestEnum.A, TestEnum.B, TestEnum.C], d: int) -> uoffset: ...




from __future__ import print_function

from concurrent import futures
import os
import sys

import flatbuffers
import grpc

sys.path.append(os.path.join(os.path.dirname(__file__), "..", "..", "tests"))
import MyGame.Example.Monster as Monster
import MyGame.Example.Stat as Stat
import MyGame.Example.Vec3 as Vec3
import MyGame.Example.Test as Test
import MyGame.Example.monster_test_grpc_fb as monster_grpc_fb


test_stat_id = "test_stat_id"
test_stat_val = 8
test_stat_count = 1

test_monster_name1 = "test_monster_name1"
test_monster_name2 = "test_monster_name2"
test_string = "test_string"
test_color = 2
test_X = 3.0
test_Y = 2.0
test_Z = 6.0
test_test1 = 4.0
test_a = 8
test_b = 5
test_hp = 67
test_inventory = [1, 1, 2, 3, 5, 8]
test_testtype = 4

test_monsters_name_retrieve = ["big_monster", "small_monster"]
test_no_of_monsters = 2


class MonsterStorage(monster_grpc_fb.MonsterStorageServicer):

  def Store(self, request, context):

    m = Monster.Monster().GetRootAsMonster(request, 0)

    assert m.Name().decode("utf-8") == test_monster_name1

    assert m.Pos().X() == test_X
    assert m.Pos().Y() == test_Y
    assert m.Pos().Z() == test_Z
    assert m.Pos().Test1() == test_test1
    assert m.Pos().Test2() == test_color
    test3 = Test.Test()
    assert m.Pos().Test3(test3).A() == test_a
    assert m.Pos().Test3(test3).B() == test_b

    assert m.Hp() == test_hp

    assert m.Color() == test_color

    assert m.InventoryLength() == len(test_inventory)
    for i in range(0, len(test_inventory)):
      assert m.Inventory(i) == test_inventory[len(test_inventory) - i - 1]

    assert m.TestType() == test_testtype

    assert m.Test() is not None
    table = m.Test()

    m2 = Monster.Monster()
    m2.Init(table.Bytes, table.Pos)
    assert m2.Name().decode("utf-8") == test_monster_name2

    m3 = m.Enemy()
    assert m3.Name().decode("utf-8") == test_monster_name2

    assert m.Testarrayofstring(0).decode("utf-8") == test_string

    b = flatbuffers.Builder(0)
    i = b.CreateString(test_stat_id)
    Stat.StatStart(b)
    Stat.StatAddId(b, i)
    Stat.StatAddVal(b, test_stat_val)
    Stat.StatAddCount(b, test_stat_count)
    b.Finish(Stat.StatEnd(b))
    return bytes(b.Output())

  def Retrieve(self, request, context):

    s = Stat.Stat().GetRootAsStat(request, 0)

    no_of_monsters = test_no_of_monsters
    for i in range(0, no_of_monsters):
      b = flatbuffers.Builder(0)
      i = b.CreateString(test_monsters_name_retrieve[i])
      Monster.MonsterStart(b)
      Monster.MonsterAddName(b, i)
      b.Finish(Monster.MonsterEnd(b))
      yield bytes(b.Output())


def serve():

  server = grpc.server(futures.ThreadPoolExecutor(max_workers=10))
  monster_grpc_fb.add_MonsterStorageServicer_to_server(MonsterStorage(), server)
  server.add_insecure_port("[::]:50051")

  server.start()

  run()


def run():

  channel = grpc.insecure_channel("127.0.0.1:50051")
  stub = monster_grpc_fb.MonsterStorageStub(channel)

  b = flatbuffers.Builder(0)
  name2 = b.CreateString(test_monster_name2)
  name1 = b.CreateString(test_monster_name1)
  Monster.MonsterStart(b)
  Monster.MonsterAddName(b, name2)
  monster2 = Monster.MonsterEnd(b)
  test1 = b.CreateString(test_string)

  Monster.MonsterStartInventoryVector(b, len(test_inventory))
  for i in range(0, len(test_inventory)):
    b.PrependByte(test_inventory[i])
  inv = b.EndVector()

  Monster.MonsterStartTest4Vector(b, 2)
  Test.CreateTest(b, 10, 20)
  Test.CreateTest(b, 30, 40)
  test4 = b.EndVector()

  Monster.MonsterStartTestarrayofstringVector(b, 1)
  b.PrependUOffsetTRelative(test1)
  test_array_of_string = b.EndVector()

  Monster.MonsterStart(b)

  Monster.MonsterAddHp(b, test_hp)
  Monster.MonsterAddName(b, name1)
  Monster.MonsterAddColor(b, test_color)
  pos = Vec3.CreateVec3(
      b, test_X, test_Y, test_Z, test_test1, test_color, test_a, test_b
  )
  Monster.MonsterAddPos(b, pos)
  Monster.MonsterAddInventory(b, inv)
  Monster.MonsterAddTestType(b, test_testtype)
  Monster.MonsterAddTest(b, monster2)
  Monster.MonsterAddTest4(b, test4)
  Monster.MonsterAddEnemy(b, monster2)
  Monster.MonsterAddTestarrayofstring(b, test_array_of_string)
  monster = Monster.MonsterEnd(b)

  b.Finish(monster)

  stat_response = stub.Store(bytes(b.Output()))

  s = Stat.Stat().GetRootAsStat(stat_response, 0)

  assert s.Id().decode("utf-8") == test_stat_id
  assert s.Val() == test_stat_val
  assert s.Count() == test_stat_count

  monster_reponses = stub.Retrieve(stat_response)
  count = 0
  for monster_reponse in monster_reponses:
    m = Monster.Monster().GetRootAsMonster(monster_reponse, 0)
    assert m.Name().decode("utf-8") == test_monsters_name_retrieve[count]
    count = count + 1


if __name__ == "__main__":
  serve()



# Copyright 2022 Google Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
from flatc_test import *


class SchemaTests:

  def EnumValAttributes(self):
    # Generate .bfbs schema first
    flatc(
        ["--schema", "--binary", "--bfbs-builtins", "enum_val_attributes.fbs"]
    )
    assert_file_exists("enum_val_attributes.bfbs")

    # Then turn it into JSON
    flatc([
        "--json",
        "--strict-json",
        str(reflection_fbs_path()),
        "--",
        "enum_val_attributes.bfbs",
    ])

    # The attributes should be present in JSON
    schema_json = json.loads(get_file_contents("enum_val_attributes.json"))

    assert schema_json["enums"][0]["name"] == "ValAttributes"
    assert schema_json["enums"][0]["values"][0]["name"] == "Val1"
    assert (
        schema_json["enums"][0]["values"][0]["attributes"][0]["key"]
        == "display_name"
    )
    assert (
        schema_json["enums"][0]["values"][0]["attributes"][0]["value"]
        == "Value 1"
    )

    assert schema_json["enums"][0]["values"][1]["name"] == "Val2"
    assert (
        schema_json["enums"][0]["values"][1]["attributes"][0]["key"]
        == "display_name"
    )
    assert (
        schema_json["enums"][0]["values"][1]["attributes"][0]["value"]
        == "Value 2"
    )

    assert schema_json["enums"][0]["values"][2]["name"] == "Val3"
    assert (
        schema_json["enums"][0]["values"][2]["attributes"][0]["key"]
        == "deprecated"
    )
    assert (
        schema_json["enums"][0]["values"][2]["attributes"][1]["key"]
        == "display_name"
    )
    assert (
        schema_json["enums"][0]["values"][2]["attributes"][1]["value"]
        == "Value 3 (deprecated)"
    )



# automatically generated by the FlatBuffers compiler, do not modify

# namespace: NamespaceA

import flatbuffers


class TableInC(object):
  __slots__ = ['_tab']

  # TableInC
  def Init(self, buf, pos):
    self._tab = flatbuffers.table.Table(buf, pos)

  # TableInC
  def ReferToA1(self):
    o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
    if o != 0:
      x = self._tab.Indirect(o + self._tab.Pos)
      from .TableInFirstNS import TableInFirstNS

      obj = TableInFirstNS()
      obj.Init(self._tab.Bytes, x)
      return obj
    return None

  # TableInC
  def ReferToA2(self):
    o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
    if o != 0:
      x = self._tab.Indirect(o + self._tab.Pos)
      from .SecondTableInA import SecondTableInA

      obj = SecondTableInA()
      obj.Init(self._tab.Bytes, x)
      return obj
    return None


def TableInCStart(builder):
  builder.StartObject(2)


def TableInCAddReferToA1(builder, referToA1):
  builder.PrependUOffsetTRelativeSlot(
      0, flatbuffers.number_types.UOffsetTFlags.py_type(referToA1), 0
  )


def TableInCAddReferToA2(builder, referToA2):
  builder.PrependUOffsetTRelativeSlot(
      1, flatbuffers.number_types.UOffsetTFlags.py_type(referToA2), 0
  )


def TableInCEnd(builder):
  return builder.EndObject()



# Copyright 2014 Google Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import collections
import struct

from . import packer
from .compat import NumpyRequiredForThisFeature, import_numpy

np = import_numpy()

# For reference, see:
# https://docs.python.org/2/library/ctypes.html#ctypes-fundamental-data-types-2

# These classes could be collections.namedtuple instances, but those are new
# in 2.6 and we want to work towards 2.5 compatability.


class BoolFlags(object):
  bytewidth = 1
  min_val = False
  max_val = True
  py_type = bool
  name = "bool"
  packer_type = packer.boolean


class Uint8Flags(object):
  bytewidth = 1
  min_val = 0
  max_val = (2**8) - 1
  py_type = int
  name = "uint8"
  packer_type = packer.uint8


class Uint16Flags(object):
  bytewidth = 2
  min_val = 0
  max_val = (2**16) - 1
  py_type = int
  name = "uint16"
  packer_type = packer.uint16


class Uint32Flags(object):
  bytewidth = 4
  min_val = 0
  max_val = (2**32) - 1
  py_type = int
  name = "uint32"
  packer_type = packer.uint32


class Uint64Flags(object):
  bytewidth = 8
  min_val = 0
  max_val = (2**64) - 1
  py_type = int
  name = "uint64"
  packer_type = packer.uint64


class Int8Flags(object):
  bytewidth = 1
  min_val = -(2**7)
  max_val = (2**7) - 1
  py_type = int
  name = "int8"
  packer_type = packer.int8


class Int16Flags(object):
  bytewidth = 2
  min_val = -(2**15)
  max_val = (2**15) - 1
  py_type = int
  name = "int16"
  packer_type = packer.int16


class Int32Flags(object):
  bytewidth = 4
  min_val = -(2**31)
  max_val = (2**31) - 1
  py_type = int
  name = "int32"
  packer_type = packer.int32


class Int64Flags(object):
  bytewidth = 8
  min_val = -(2**63)
  max_val = (2**63) - 1
  py_type = int
  name = "int64"
  packer_type = packer.int64


class Float32Flags(object):
  bytewidth = 4
  min_val = None
  max_val = None
  py_type = float
  name = "float32"
  packer_type = packer.float32


class Float64Flags(object):
  bytewidth = 8
  min_val = None
  max_val = None
  py_type = float
  name = "float64"
  packer_type = packer.float64


class SOffsetTFlags(Int32Flags):
  pass


class UOffsetTFlags(Uint32Flags):
  pass


class VOffsetTFlags(Uint16Flags):
  pass


def valid_number(n, flags):
  if flags.min_val is None and flags.max_val is None:
    return True
  return flags.min_val <= n <= flags.max_val


def enforce_number(n, flags):
  if flags.min_val is None and flags.max_val is None:
    return
  if not flags.min_val <= n <= flags.max_val:
    raise TypeError("bad number %s for type %s" % (str(n), flags.name))


def float32_to_uint32(n):
  packed = struct.pack("<1f", n)
  (converted,) = struct.unpack("<1L", packed)
  return converted


def uint32_to_float32(n):
  packed = struct.pack("<1L", n)
  (unpacked,) = struct.unpack("<1f", packed)
  return unpacked


def float64_to_uint64(n):
  packed = struct.pack("<1d", n)
  (converted,) = struct.unpack("<1Q", packed)
  return converted


def uint64_to_float64(n):
  packed = struct.pack("<1Q", n)
  (unpacked,) = struct.unpack("<1d", packed)
  return unpacked


def to_numpy_type(number_type):
  if np is not None:
    return np.dtype(number_type.name).newbyteorder("<")
  else:
    raise NumpyRequiredForThisFeature("Numpy was not found.")



#!/usr/bin/env python3
#
# Copyright 2022 Google Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pathlib import Path
import platform
import shutil
import subprocess
import sys

# Get the path where this script is located so we can invoke the script from
# any directory and have the paths work correctly.
tests_path = Path(__file__).parent.resolve()

# Get the root path as an absolute path, so all derived paths are absolute.
root_path = tests_path.parent.parent.absolute()

# Windows works with subprocess.run a bit differently.
is_windows = platform.system() == "Windows"

# Get the location of the flatc executable
flatc_exe = Path("flatc.exe" if is_windows else "flatc")

# Find and assert flatc compiler is present.
if root_path in flatc_exe.parents:
  flatc_exe = flatc_exe.relative_to(root_path)
flatc_path = Path(root_path, flatc_exe)
assert flatc_path.exists(), "Cannot find the flatc compiler " + str(flatc_path)


def check_call(args, cwd=tests_path):
  subprocess.check_call(args, cwd=str(cwd), shell=is_windows)


# Execute the flatc compiler with the specified parameters
def flatc(
    options, schema, prefix=None, include=None, data=None, cwd=tests_path
):
  print("Invoking flatc on schema " + str(schema))
  cmd = [str(flatc_path)] + options
  if prefix:
    cmd += ["-o"] + [prefix]
  if include:
    cmd += ["-I"] + [include]
  cmd += [schema] if isinstance(schema, str) else schema
  if data:
    cmd += [data] if isinstance(data, str) else data
  check_call(cmd)


# Execute esbuild with the specified parameters
def esbuild(input, output):
  cmd = ["esbuild", input, "--outfile=" + output]
  cmd += ["--format=cjs", "--bundle", "--external:flatbuffers"]
  check_call(cmd)


print("Removing node_modules/ directory...")
shutil.rmtree(Path(tests_path, "node_modules"), ignore_errors=True)

check_call(["npm", "install", "--silent"])

flatc(
    options=[
        "--ts",
        "--reflect-names",
        "--gen-name-strings",
        "--gen-mutable",
        "--gen-object-api",
        "--ts-entry-points",
        "--ts-flat-files",
    ],
    schema="../monster_test.fbs",
    include="../include_test",
)
esbuild("monster_test.ts", "monster_test_generated.cjs")

flatc(
    options=["--gen-object-api", "-b"],
    schema="../monster_test.fbs",
    include="../include_test",
    data="../unicode_test.json",
)

flatc(
    options=[
        "--ts",
        "--reflect-names",
        "--gen-name-strings",
        "--gen-mutable",
        "--gen-object-api",
        "--ts-entry-points",
        "--ts-flat-files",
    ],
    schema="../union_vector/union_vector.fbs",
    prefix="union_vector",
)
esbuild(
    "union_vector/union_vector.ts", "union_vector/union_vector_generated.cjs"
)

flatc(
    options=["--ts", "--reflect-names", "--gen-name-strings"],
    schema="../optional_scalars.fbs",
)

flatc(
    options=[
        "--ts",
        "--reflect-names",
        "--gen-name-strings",
        "--ts-no-import-ext",
    ],
    schema="../optional_scalars.fbs",
    prefix="no_import_ext",
)

flatc(
    options=[
        "--ts",
        "--reflect-names",
        "--gen-name-strings",
        "--gen-object-api",
        "--ts-entry-points",
        "--ts-flat-files",
    ],
    schema="arrays_test_complex/arrays_test_complex.fbs",
    prefix="arrays_test_complex",
)
esbuild(
    "arrays_test_complex/my-game/example.ts",
    "arrays_test_complex/arrays_test_complex_generated.cjs",
)

flatc(
    options=[
        "--ts",
        "--reflect-names",
        "--gen-name-strings",
        "--gen-mutable",
        "--gen-object-api",
        "--ts-entry-points",
        "--ts-flat-files",
    ],
    schema=[
        "typescript_keywords.fbs",
        "test_dir/typescript_include.fbs",
        "test_dir/typescript_transitive_include.fbs",
        "../../reflection/reflection.fbs",
    ],
    include="../../",
)
esbuild("typescript_keywords.ts", "typescript_keywords_generated.cjs")

flatc(
    options=[
        "--ts",
        "--reflect-names",
        "--gen-name-strings",
        "--gen-mutable",
        "--gen-object-api",
        "--ts-entry-points",
        "--ts-flat-files",
    ],
    schema="../union_underlying_type_test.fbs",
)

flatc(options=["--ts"], schema="../long_namespace.fbs")
flatc(options=["--ts"], schema="../longer_namespace.fbs")

print("Running TypeScript Compiler...")
check_call(["tsc"])
print(
    "Running TypeScript Compiler in old node resolution mode for"
    " no_import_ext..."
)
check_call(["tsc", "-p", "./tsconfig.node.json"])

NODE_CMD = ["node"]

print("Running TypeScript Tests...")
check_call(NODE_CMD + ["JavaScriptTest"])
check_call(NODE_CMD + ["JavaScriptUnionVectorTest"])
check_call(NODE_CMD + ["JavaScriptFlexBuffersTest"])
check_call(NODE_CMD + ["JavaScriptComplexArraysTest"])
check_call(NODE_CMD + ["JavaScriptUnionUnderlyingTypeTest"])

print("Running old v1 TypeScript Tests...")
check_call(NODE_CMD + ["JavaScriptTestv1.cjs", "./monster_test_generated.cjs"])




def print_tables(tables, prefix='zlmdb', inherit=False):
    tables = sorted(tables)

    for table in tables:
        print('* :class:`{}.{}`'.format(prefix, table))

    print('')
    print('------')
    print('')

    tmpl = """.. autoclass:: {}.{}
        :members:
    """
    if inherit:
        tmpl += "    :show-inheritance:"

    for table in tables:
        print(tmpl.format(prefix, table))


from zlmdb import __all__ as a1
print_tables([x for x in a1 if x.startswith('Map')], inherit=True)

from zlmdb._types import __dict__ as a2
print_tables([x for x in a2 if x.endswith('KeysMixin')], prefix='zlmdb._types')
print_tables([x for x in a2 if x.endswith('ValuesMixin')], prefix='zlmdb._types')



from __future__ import annotations

import flatbuffers
import numpy as np

import typing
from MyGame.Example.NestedUnion.TestSimpleTableWithEnum import TestSimpleTableWithEnum
from MyGame.Example.NestedUnion.Vec3 import Vec3
from flatbuffers import table
from typing import cast

uoffset: typing.TypeAlias = flatbuffers.number_types.UOffsetTFlags.py_type

class Any(object):
  NONE = cast(int, ...)
  Vec3 = cast(int, ...)
  TestSimpleTableWithEnum = cast(int, ...)
def AnyCreator(union_type: typing.Literal[Any.NONE, Any.Vec3, Any.TestSimpleTableWithEnum], table: table.Table) -> typing.Union[None, Vec3, TestSimpleTableWithEnum]: ...




# automatically generated by the FlatBuffers compiler, do not modify

# namespace: Example

import flatbuffers
from flatbuffers.compat import import_numpy
np = import_numpy()

class StructOfStructsOfStructs(object):
    __slots__ = ['_tab']

    @classmethod
    def SizeOf(cls):
        return 20

    # StructOfStructsOfStructs
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # StructOfStructsOfStructs
    def A(self, obj):
        obj.Init(self._tab.Bytes, self._tab.Pos + 0)
        return obj


def CreateStructOfStructsOfStructs(builder, a_a_id, a_a_distance, a_b_a, a_b_b, a_c_id, a_c_distance):
    builder.Prep(4, 20)
    builder.Prep(4, 20)
    builder.Prep(4, 8)
    builder.PrependUint32(a_c_distance)
    builder.PrependUint32(a_c_id)
    builder.Prep(2, 4)
    builder.Pad(1)
    builder.PrependInt8(a_b_b)
    builder.PrependInt16(a_b_a)
    builder.Prep(4, 8)
    builder.PrependUint32(a_a_distance)
    builder.PrependUint32(a_a_id)
    return builder.Offset()

import MyGame.Example.StructOfStructs
try:
    from typing import Optional
except:
    pass

class StructOfStructsOfStructsT(object):

    # StructOfStructsOfStructsT
    def __init__(
        self,
        a = None,
    ):
        self.a = a  # type: Optional[MyGame.Example.StructOfStructs.StructOfStructsT]

    @classmethod
    def InitFromBuf(cls, buf, pos):
        structOfStructsOfStructs = StructOfStructsOfStructs()
        structOfStructsOfStructs.Init(buf, pos)
        return cls.InitFromObj(structOfStructsOfStructs)

    @classmethod
    def InitFromPackedBuf(cls, buf, pos=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, pos)
        return cls.InitFromBuf(buf, pos+n)

    @classmethod
    def InitFromObj(cls, structOfStructsOfStructs):
        x = StructOfStructsOfStructsT()
        x._UnPack(structOfStructsOfStructs)
        return x

    # StructOfStructsOfStructsT
    def _UnPack(self, structOfStructsOfStructs):
        if structOfStructsOfStructs is None:
            return
        if structOfStructsOfStructs.A(MyGame.Example.StructOfStructs.StructOfStructs()) is not None:
            self.a = MyGame.Example.StructOfStructs.StructOfStructsT.InitFromObj(structOfStructsOfStructs.A(MyGame.Example.StructOfStructs.StructOfStructs()))

    # StructOfStructsOfStructsT
    def Pack(self, builder):
        return CreateStructOfStructsOfStructs(builder, self.a.a.id, self.a.a.distance, self.a.b.a, self.a.b.b, self.a.c.id, self.a.c.distance)



from __future__ import annotations

import flatbuffers
import numpy as np

import typing
from typing import cast

uoffset: typing.TypeAlias = flatbuffers.number_types.UOffsetTFlags.py_type

class Color(object):
  Red = cast(int, ...)
  Green = cast(int, ...)
  Blue = cast(int, ...)




# Copyright 2013-2025 The py-lmdb authors, all rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted only as authorized by the OpenLDAP
# Public License.
#
# A copy of this license is available in the file LICENSE in the
# top-level directory of the distribution or, alternatively, at
# <http://www.OpenLDAP.org/license.html>.
#
# OpenLDAP is a registered trademark of the OpenLDAP Foundation.
#
# Individual files and/or contributed packages may be copyright by
# other parties and/or subject to additional restrictions.
#
# This work also contains materials derived from public sources.
#
# Additional information about OpenLDAP can be obtained at
# <http://www.openldap.org/>.

# Roughly approximates some of Symas microbenchmark.

from time import time
import random
import shutil
import os
import tempfile

import zlmdb.lmdb as lmdb


val = b" " * 100
MAX_KEYS = int(1e6)

t0 = time()

urandom = open("/dev/urandom", "rb", 1048576).read

keys = set()
while len(keys) < MAX_KEYS:
    for _ in range(min(1000, MAX_KEYS - len(keys))):
        keys.add(urandom(16))

print("make %d keys in %.2fsec" % (len(keys), time() - t0))
keys = list(keys)

if os.path.exists("/ram"):
    DB_PATH = "/ram/dbtest"
else:
    DB_PATH = tempfile.mktemp(prefix="nastybench")

if os.path.exists(DB_PATH):
    shutil.rmtree(DB_PATH)

env = lmdb.open(
    DB_PATH, map_size=1048576 * 1024, metasync=False, sync=False, map_async=True
)

nextkey = iter(keys).__next__
run = True
while run:
    with env.begin(write=True) as txn:
        try:
            for _ in range(10000):
                txn.put(nextkey(), val)
        except StopIteration:
            run = False

d = time() - t0
env.sync(True)
print("insert %d keys in %.2fsec (%d/sec)" % (len(keys), d, len(keys) / d))


nextkey = iter(keys).__next__
t0 = time()

with env.begin() as txn:
    try:
        while 1:
            txn.get(nextkey())
    except StopIteration:
        pass

d = time() - t0
print("random lookup %d keys in %.2fsec (%d/sec)" % (len(keys), d, len(keys) / d))


nextkey = iter(keys).__next__
t0 = time()

with env.begin(buffers=True) as txn:
    try:
        while 1:
            txn.get(nextkey())
    except StopIteration:
        pass

d = time() - t0
print("random lookup %d buffers in %.2fsec (%d/sec)" % (len(keys), d, len(keys) / d))


nextkey = iter(keys).__next__
t0 = time()

with env.begin(buffers=True) as txn:
    try:
        while 1:
            hash(bytes(txn.get(nextkey())))
    except StopIteration:
        pass

d = time() - t0
print(
    "random lookup+hash %d buffers in %.2fsec (%d/sec)" % (len(keys), d, len(keys) / d)
)


nextkey = iter(keys).__next__
t0 = time()

with env.begin(buffers=True) as txn:
    nextrec = txn.cursor().iternext().__next__
    try:
        while 1:
            nextrec()
    except StopIteration:
        pass

d = time() - t0
print("seq read %d buffers in %.2fsec (%d/sec)" % (len(keys), d, len(keys) / d))



from pathlib import Path
import sys

# Get the path where this script is located so we can invoke the script from
# any directory and have the paths work correctly.
script_path = Path(__file__).parent.resolve()

# Get the root path as an absolute path, so all derived paths are absolute.
root_path = script_path.parent.absolute()

# Get the location of the schema
schema_path = Path(script_path, "schema")

# Too add the util package in /scripts/util.py
sys.path.append(str(root_path.absolute()))

from scripts.util import flatc


def flatc_golden(options, schema, prefix):
  # wrap the generic flatc call with specifis for these goldens.
  flatc(
      options=options,
      # where the files are generated, typically the language (e.g. "cpp").
      prefix=prefix,
      # The schema are relative to the schema directory.
      schema=str(Path(schema_path, schema)),
      # Run flatc from this location.
      cwd=script_path,
  )



# automatically generated by the FlatBuffers compiler, do not modify

# namespace: example

from typing import Any
import flatbuffers


class HelloRequest(object):
  __slots__ = ['_tab']

  @classmethod
  def GetRootAs(cls, buf, offset: int = 0):
    n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
    x = HelloRequest()
    x.Init(buf, n + offset)
    return x

  @classmethod
  def GetRootAsHelloRequest(cls, buf, offset=0):
    """This method is deprecated. Please switch to GetRootAs."""
    return cls.GetRootAs(buf, offset)

  # HelloRequest
  def Init(self, buf: bytes, pos: int):
    self._tab = flatbuffers.table.Table(buf, pos)


def HelloRequestStart(builder: flatbuffers.Builder):
  builder.StartObject(0)


def HelloRequestEnd(builder: flatbuffers.Builder) -> int:
  return builder.EndObject()


class HelloResponse(object):
  __slots__ = ['_tab']

  @classmethod
  def GetRootAs(cls, buf, offset: int = 0):
    n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
    x = HelloResponse()
    x.Init(buf, n + offset)
    return x

  @classmethod
  def GetRootAsHelloResponse(cls, buf, offset=0):
    """This method is deprecated. Please switch to GetRootAs."""
    return cls.GetRootAs(buf, offset)

  # HelloResponse
  def Init(self, buf: bytes, pos: int):
    self._tab = flatbuffers.table.Table(buf, pos)


def HelloResponseStart(builder: flatbuffers.Builder):
  builder.StartObject(0)


def HelloResponseEnd(builder: flatbuffers.Builder) -> int:
  return builder.EndObject()



# automatically generated by the FlatBuffers compiler, do not modify

# namespace: Example

import flatbuffers
from flatbuffers.compat import import_numpy
from typing import Any
from MyGame.Example.NestedStruct import NestedStruct
np = import_numpy()

class ArrayStruct(object):
    __slots__ = ['_tab']

    @classmethod
    def SizeOf(cls) -> int:
        return 160

    # ArrayStruct
    def Init(self, buf: bytes, pos: int):
        self._tab = flatbuffers.table.Table(buf, pos)

    # ArrayStruct
    def A(self): return self._tab.Get(flatbuffers.number_types.Float32Flags, self._tab.Pos + flatbuffers.number_types.UOffsetTFlags.py_type(0))
    # ArrayStruct
    def B(self, j = None):
        if j is None:
            return [self._tab.Get(flatbuffers.number_types.Int32Flags, self._tab.Pos + flatbuffers.number_types.UOffsetTFlags.py_type(4 + i * 4)) for i in range(self.BLength())]
        elif j >= 0 and j < self.BLength():
            return self._tab.Get(flatbuffers.number_types.Int32Flags, self._tab.Pos + flatbuffers.number_types.UOffsetTFlags.py_type(4 + j * 4))
        else:
            return None

    # ArrayStruct
    def BAsNumpy(self):
        return self._tab.GetArrayAsNumpy(flatbuffers.number_types.Int32Flags, self._tab.Pos + 4, self.BLength())

    # ArrayStruct
    def BLength(self) -> int:
        return 15

    # ArrayStruct
    def BIsNone(self) -> bool:
        return False

    # ArrayStruct
    def C(self): return self._tab.Get(flatbuffers.number_types.Int8Flags, self._tab.Pos + flatbuffers.number_types.UOffsetTFlags.py_type(64))
    # ArrayStruct
    def D(self, i: int) -> NestedStruct:
        obj = NestedStruct()
        obj.Init(self._tab.Bytes, self._tab.Pos + 72 + i * 32)
        return obj

    # ArrayStruct
    def DLength(self) -> int:
        return 2

    # ArrayStruct
    def DIsNone(self) -> bool:
        return False

    # ArrayStruct
    def E(self): return self._tab.Get(flatbuffers.number_types.Int32Flags, self._tab.Pos + flatbuffers.number_types.UOffsetTFlags.py_type(136))
    # ArrayStruct
    def F(self, j = None):
        if j is None:
            return [self._tab.Get(flatbuffers.number_types.Int64Flags, self._tab.Pos + flatbuffers.number_types.UOffsetTFlags.py_type(144 + i * 8)) for i in range(self.FLength())]
        elif j >= 0 and j < self.FLength():
            return self._tab.Get(flatbuffers.number_types.Int64Flags, self._tab.Pos + flatbuffers.number_types.UOffsetTFlags.py_type(144 + j * 8))
        else:
            return None

    # ArrayStruct
    def FAsNumpy(self):
        return self._tab.GetArrayAsNumpy(flatbuffers.number_types.Int64Flags, self._tab.Pos + 144, self.FLength())

    # ArrayStruct
    def FLength(self) -> int:
        return 2

    # ArrayStruct
    def FIsNone(self) -> bool:
        return False


def CreateArrayStruct(builder, a, b, c, d_a, d_b, d_c, d_d, e, f):
    builder.Prep(8, 160)
    for _idx0 in range(2 , 0, -1):
        builder.PrependInt64(f[_idx0-1])
    builder.Pad(4)
    builder.PrependInt32(e)
    for _idx0 in range(2 , 0, -1):
        builder.Prep(8, 32)
        for _idx1 in range(2 , 0, -1):
            builder.PrependInt64(d_d[_idx0-1][_idx1-1])
        builder.Pad(5)
        for _idx1 in range(2 , 0, -1):
            builder.PrependInt8(d_c[_idx0-1][_idx1-1])
        builder.PrependInt8(d_b[_idx0-1])
        for _idx1 in range(2 , 0, -1):
            builder.PrependInt32(d_a[_idx0-1][_idx1-1])
    builder.Pad(7)
    builder.PrependInt8(c)
    for _idx0 in range(15 , 0, -1):
        builder.PrependInt32(b[_idx0-1])
    builder.PrependFloat32(a)
    return builder.Offset()

import MyGame.Example.NestedStruct
try:
    from typing import List
except:
    pass

class ArrayStructT(object):

    # ArrayStructT
    def __init__(
        self,
        a = 0.0,
        b = None,
        c = 0,
        d = None,
        e = 0,
        f = None,
    ):
        self.a = a  # type: float
        self.b = b  # type: Optional[List[int]]
        self.c = c  # type: int
        self.d = d  # type: Optional[List[MyGame.Example.NestedStruct.NestedStructT]]
        self.e = e  # type: int
        self.f = f  # type: Optional[List[int]]

    @classmethod
    def InitFromBuf(cls, buf, pos):
        arrayStruct = ArrayStruct()
        arrayStruct.Init(buf, pos)
        return cls.InitFromObj(arrayStruct)

    @classmethod
    def InitFromPackedBuf(cls, buf, pos=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, pos)
        return cls.InitFromBuf(buf, pos+n)

    @classmethod
    def InitFromObj(cls, arrayStruct):
        x = ArrayStructT()
        x._UnPack(arrayStruct)
        return x

    # ArrayStructT
    def _UnPack(self, arrayStruct):
        if arrayStruct is None:
            return
        self.a = arrayStruct.A()
        if not arrayStruct.BIsNone():
            if np is None:
                self.b = []
                for i in range(arrayStruct.BLength()):
                    self.b.append(arrayStruct.B(i))
            else:
                self.b = arrayStruct.BAsNumpy()
        self.c = arrayStruct.C()
        if not arrayStruct.DIsNone():
            self.d = []
            for i in range(arrayStruct.DLength()):
                if arrayStruct.D(i) is None:
                    self.d.append(None)
                else:
                    nestedStruct_ = MyGame.Example.NestedStruct.NestedStructT.InitFromObj(arrayStruct.D(i))
                    self.d.append(nestedStruct_)
        self.e = arrayStruct.E()
        if not arrayStruct.FIsNone():
            if np is None:
                self.f = []
                for i in range(arrayStruct.FLength()):
                    self.f.append(arrayStruct.F(i))
            else:
                self.f = arrayStruct.FAsNumpy()

    # ArrayStructT
    def Pack(self, builder):
        return CreateArrayStruct(builder, self.a, self.b, self.c, self.d.a, self.d.b, self.d.c, self.d.d, self.e, self.f)



# automatically generated by the FlatBuffers compiler, do not modify

# namespace: NamespaceB


class UnionInNestedNS(object):
  NONE = 0
  TableInNestedNS = 1


def UnionInNestedNSCreator(unionType, table):
  from flatbuffers.table import Table

  if not isinstance(table, Table):
    return None
  if unionType == UnionInNestedNS().TableInNestedNS:
    return TableInNestedNST.InitFromBuf(table.Bytes, table.Pos)
  return None



from __future__ import annotations

import flatbuffers
import numpy as np

import typing
from MyGame.Example.NestedStruct import NestedStruct, NestedStructT
from MyGame.Example.TestEnum import TestEnum

uoffset: typing.TypeAlias = flatbuffers.number_types.UOffsetTFlags.py_type

class ArrayStruct(object):
  @classmethod
  def SizeOf(cls) -> int: ...

  def Init(self, buf: bytes, pos: int) -> None: ...
  def A(self) -> float: ...
  def B(self, i: int) -> typing.List[int]: ...
  def BAsNumpy(self) -> np.ndarray: ...
  def BLength(self) -> int: ...
  def BIsNone(self) -> bool: ...
  def C(self) -> int: ...
  def D(self, i: int) -> NestedStruct | None: ...
  def DLength(self) -> int: ...
  def DIsNone(self) -> bool: ...
  def E(self) -> int: ...
  def F(self, i: int) -> typing.List[int]: ...
  def FAsNumpy(self) -> np.ndarray: ...
  def FLength(self) -> int: ...
  def FIsNone(self) -> bool: ...
class ArrayStructT(object):
  a: float
  b: typing.List[int]
  c: int
  d: typing.List[NestedStructT]
  e: int
  f: typing.List[int]
  def __init__(
    self,
    a: float = ...,
    b: typing.List[int] | None = ...,
    c: int = ...,
    d: typing.List['NestedStructT'] | None = ...,
    e: int = ...,
    f: typing.List[int] | None = ...,
  ) -> None: ...
  @classmethod
  def InitFromBuf(cls, buf: bytes, pos: int) -> ArrayStructT: ...
  @classmethod
  def InitFromPackedBuf(cls, buf: bytes, pos: int = 0) -> ArrayStructT: ...
  @classmethod
  def InitFromObj(cls, arrayStruct: ArrayStruct) -> ArrayStructT: ...
  def _UnPack(self, arrayStruct: ArrayStruct) -> None: ...
  def Pack(self, builder: flatbuffers.Builder) -> None: ...

def CreateArrayStruct(builder: flatbuffers.Builder, a: float, b: int, c: int, d_a: int, d_b: typing.Literal[TestEnum.A, TestEnum.B, TestEnum.C], d_c: typing.Literal[TestEnum.A, TestEnum.B, TestEnum.C], d_d: int, e: int, f: int) -> uoffset: ...




import glob
import os
from pathlib import Path
import shutil
import subprocess

test_nim_dir = Path(__file__).absolute().parent
test_dir = test_nim_dir.parent


def main():
  try:
    subprocess.check_call("testament --megatest:off all".split())
  finally:
    shutil.rmtree(test_nim_dir / "nimcache")
    shutil.rmtree(test_nim_dir / "testresults")
    for f in glob.glob(str(test_nim_dir / "tests" / "*" / "test")):
      os.remove(f)


if __name__ == "__main__":
  main()
